package me.icymint.libra.sas.config;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.registry.RegistryKey;
import com.registry.RegistryValue;

/**
 * 系统配置。
 * 
 * @author Daniel
 * @datetime 2012-10-22下午2:48:54
 * 
 */
public class SasSystemConfig extends Properties {

	/**
	 * 
	 */
	private static final long serialVersionUID = -1192762076079791662L;
	protected final Logger logger = LoggerFactory.getLogger(getClass());
	private final String name;

	public SasSystemConfig() throws IOException {
		this("SDE", null);
	}

	public SasSystemConfig(String name, InputStream is) throws IOException {
		this.name = name;

		/**
		 * 内部初始化。
		 */
		this.load(getClass().getResourceAsStream("SDE.properties"));

		/**
		 * SDE外部初始化。
		 */
		String root = System.getProperty("usr.dir");
		if (!this.replaceLoad(new File(root, "SDE.properties"))) {
			this.replaceLoad(new File(new File(root, name), "SDE.properties"));
		}

		/**
		 * 自定义初始化。
		 */
		if (is != null) {
			this.load(is);
		}

		/**
		 * 一些基本常量的初始化。
		 */
		this.put("G_ROOT", new File(root, name).getAbsolutePath());
		if (this.getProperty("G_CHECK_TYPE") == "0")
			this.setProperty("G_CHECK_TYPE", ",");
		else
			this.setProperty("G_CHECK_TYPE", ",1");
	}

	/**
	 * 获取SDE目录。
	 */
	public String getName() {
		return name;
	}

	/**
	 * 获取SAS.EXE程序路径。
	 */
	public String getSasExePath() {
		RegistryKey rk = RegistryKey
				.getRootKeyForIndex(RegistryKey.HKEY_CLASSES_ROOT_INDEX);
		RegistryKey srk = new RegistryKey(rk,
				"SAS.Application\\shell\\Open\\command");
		if (srk.hasValues()) {
			RegistryValue v = srk.getValue("");
			if (v != null)
				return v.toString().substring(
						new String("Name:  Type: REG_SZ Value: ").length());
		}
		return this.getProperty("SAS");
	}

	private boolean replaceLoad(File in) throws IOException {
		try {
			this.load(new FileInputStream(in));
		} catch (FileNotFoundException e) {
			return false;
		}
		return true;
	}
}
