package me.icymint.libra.sas.session;

import java.io.File;
import java.io.IOException;

import me.icymint.libra.sas.config.SasConfig;
import me.icymint.libra.sas.exception.SasAbortException;
import me.icymint.libra.sas.exception.SasCacheFileLockedException;
import me.icymint.libra.sas.exception.SasInterruptException;
import me.icymint.libra.sas.exception.SasNotRunException;
import me.icymint.libra.sas.exception.SasRunErrorException;
import me.icymint.libra.sas.scanner.AbstractScanner;
import me.icymint.libra.sas.task.TaskScanner;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SessionScanner extends AbstractScanner<String, SessionConfig>
		implements Runnable {

	protected final Logger logger = LoggerFactory.getLogger(getClass());
	private boolean shutdown = false;
	private Process p = null;
	private final String name;

	public SessionScanner(File session, String name) {
		super(new SessionConfig(session));
		this.name = name;
	}

	@Override
	public void finallizeScanner(SessionConfig config)
			throws SasRunErrorException {
		logger.debug(getName() + "运行结束");
		super.finallizeScanner(config);
	}

	public String getName() {
		return name;
	}

	@Override
	public void initializeScanner(SessionConfig config)
			throws SasNotRunException, SasCacheFileLockedException {
		super.initializeScanner(config);
		config.getTaskCache().mkdirs();
		try {
			p = Runtime.getRuntime().exec(
					config.getSystemConfig().getSasExePath()
							+ " -NOSPLASH -ICON -SYSIN \""
							+ config.getSasFile() + "\" -log \""
							+ config.getMainLog() + "\"");
		} catch (IOException e) {
			throw new SasNotRunException();
		}
		while (!config.isStarted()) {
			synchronized (p) {
				try {
					p.wait(10);
				} catch (InterruptedException e) {
				}
			}
		}
		logger.debug(getName() + "开始运行...");
	}

	/**
	 * 本方法用于加载新的任务{@link TaskScanner}
	 * 
	 * @throws InterruptedException
	 * @throws SasCacheFileLockedException
	 * @throws SasNotRunException
	 * @throws SasAbortException
	 * @throws SasRunErrorException
	 * @throws SasInterruptException
	 */
	protected void invoke(SasConfig config) throws InterruptedException,
			SasInterruptException, SasRunErrorException, SasAbortException,
			SasCacheFileLockedException {

	}

	@Override
	public void run() {
		try {
			this.execute();
		} catch (SasRunErrorException e) {
			logger.error("SAS运行出错！");
			e.printStackTrace();
		} catch (SasInterruptException e) {
			logger.error("SAS被安全终止！");
			e.printStackTrace();
		} catch (SasAbortException e) {
			logger.error("SAS异常中断！");
			e.printStackTrace();
		} catch (SasNotRunException e) {
			logger.error("SAS未运行！");
			e.printStackTrace();
		} catch (SasCacheFileLockedException e) {
			logger.error("SAS指定缓存文件夹被锁定。");
			e.printStackTrace();
		}
	}

	@Override
	public String sas() {
		return "SDK_SAS_LOADER";
	}

	@Override
	public void scan(SessionConfig config) throws SasRunErrorException,
			SasAbortException {
		/**
		 * 加载任务。
		 */
		try {
			invoke(config);
		} catch (InterruptedException e1) {
			shutdown();
		} catch (SasInterruptException e) {
			/**
			 * 该选项有待评估，需要合适的策略来进行中断SAS会话或者继续SAS会话。
			 */
		} catch (SasCacheFileLockedException e) {
			logger.error(e.getMessage() + "\n该任务停止运行。");
		} catch (SasRunErrorException e) {
			/**
			 * 该选项有待评估，需要合适的策略来进行中断SAS会话或者继续SAS会话。
			 */
			logger.error("子过程运行有误！ ");
			e.printErrorLog();
		}
		/**
		 * 控制停止。
		 */
		if (shutdown)
			config.shutdown();
		/**
		 * 检测自动停止。
		 */
		try {
			p.exitValue();
		} catch (IllegalThreadStateException e) {
			return;
		}

		if (!config.isFinished()) {
			/**
			 * 设立运行标志。
			 */
			try {
				config.getFinishFile().createNewFile();
			} catch (IOException e) {

			}
			throw new SasAbortException();
		}
	}

	/**
	 * 停止运行。
	 */
	@Override
	public void shutdown() {
		shutdown = true;
		super.shutdown();
	}

}
